rm(list=ls())
work.dir<-"/home/aml452/src/Sim-Cages"
save.dir<-"/home/aml452/res/Sim-Cages"
setwd(work.dir)

############################

#### FUNCTIONS #############

#load functions evolve and rho.f 
source("functions/evolve-autosom.R")
source("functions/rho-conditioned.R")

#logL: set selection parameters and calculate Likelihood of observed allele frequencies, given parameters and frequencies of the previous generation 

#models:
#(d .... drive allele)
#s ... sexual selection, recessive locus
#f ... fecundity, recessive locus
#v ... viability, recessive locus
#s.simple ... sexual selection, recessive locus, one parameter
#f.simple ... fecundity, recessive locus, one parameter,multiplicative
#v.simple ... viability, recessive locus, one parameter
#f.simple2 ... fecundity, fitness reduction dd females 

logL <- function(f,N,mode) {
#set everything neutral
  v.m<-rep(1,2)
  v.f<-rep(1,2)
  w<-matrix(rep(1,9),ncol=3)
  alpha<-rep(1,2)
  beta<-rep(1,2)
  gamma<-rep(1,2)
  
  #different models
  switch(mode,
         "s" = {
            n.e<-f[1]
            alpha[2]<-f[2]
            beta[2]<-alpha[2]
            gamma[2]<-f[3]
          
         },
         
         "f" = {
           n.e<-f[1]
           w[3,1]<-f[2]
           w[1,3]<-f[3]
           w[3,3]<-w[1,3]*w[3,1]
           w[2,3]<-w[1,3]
           w[3,2]<-w[3,1]
         },
         
         "v" = {
           n.e<-f[1]
           v.m[2]<-f[2]
           v.f[2]<-f[3]
         }, 
         
         "v.simple"= { 
           n.e<-f[1]
           v.m[2]<-f[2]
           v.f[2]<-v.m[2]
           },
         
         "f.simple"= { 
           n.e<-f[1]
           w[3,1]<-f[2]
           w[1,3]<-w[3,1]
           w[3,3]<-w[1,3]*w[3,1]
           w[2,3]<-w[1,3]
           w[3,2]<-w[3,1]
           },
         
         "f.simple2"={
           n.e<-f[1]
           w[1,3]<-f[2]
           w[2,3]<-w[1,3]
           w[3,3]<-w[1,3]
         },
         
         "s.simple"= {
           n.e<-f[1]
           alpha[2]<-f[2]
           beta[2]<-alpha[2]
           gamma[2]<-alpha[2]
           }
         
         )
  
  l<-0

  for(t in c(2:nrow(N))){
    # p: observed genotype frequencies in generation t-1
    p <- N[t-1,]
    p.m <- sum(p[1:3])
    p.f <- sum(p[4:6])
    
    p[1:3]<-p[1:3]/p.m
    p[4:6]<-p[4:6]/p.f
    
    
    # x: observed genotype frequencies in generation t
    x    <- N[t,]
    x.m  <- sum(x[1:3])
    x.f  <- sum(x[4:6])
    
    x[1:3]<-x[1:3]/x.m
    x[4:6]<-x[4:6]/x.f
    
    #F:M = 1
    n.e.m <- n.e/2
    n.e.f <- n.e/2
    
    #expected genotype frequencies in generation t
    p.exp<-evolve(x=p,n.e=0,v.m = v.m,v.f = v.f,w = w,alpha = alpha,beta = beta,gamma = gamma,g = 1)
    p.exp<-p.exp[2,]
    l<-l+rho.f(x[3],x[2],p.exp[3],p.exp[2],n.e.m)+rho.f(x[6],x[5],p.exp[6],p.exp[5],n.e.f)}
    return(l)
}

#logL.all : apply function logL over all provided evolution replicates
logL.all<-function(f,N,mode){
  #N list of evolution replicates
  l<-logL(f = f,mode = mode,N = N[[1]])
  if(length(N)>1){
    for(i in c(2:length(N))) l<-l+logL(f = f,N = N[[i]],mode = mode)
  }
return(l)
}

#calculate CI based on FI 
#https://stats.stackexchange.com/questions/27033/in-r-given-an-output-from-optim-with-a-hessian-matrix-how-to-calculate-paramet
calculate.ci<-function(my.hessian){
  if(det(my.hessian)==0){
    return(NA)
  }
  fisher<-solve(c(-1)*my.hessian)
  sigma_estimate<-sqrt(diag(fisher))
  return(sigma_estimate*1.96)
}

#cross: function to generate simulation data and apply ML 
#x ... starting frequency 
#n.e ... effective population size 
#v.m .... viability, dd males 
#v.f ... viability, dd females 
#alpha ... sexual selection parameter 
#beta ... sexual selection parameter
#gamma ... sexual selection parameter 
#g ... number of generations 
#n.rep ... number of evolution replicates 
#info ... variable to store applied type of selection
#w ... fecundity matrix (3x3)
#sim.nr .... number of simulations/parameter set
#census ... census size of the population


cross<-function(x,n.e,v.m,v.f,alpha,beta,gamma,g,n.rep,info,w,sim.nr,census=1000){
  #simulate data 
  N<-replicate(n.rep,evolve(x=x,n.e=n.e,v.m = v.m,v.f = v.f,alpha = alpha,beta = beta,gamma=gamma,g = g,w=w),simplify = F)
  N<-lapply(N,function(x) x*census)
  
  #set start + limits for ML, 3 parameters estimated (Ne + 2 selection parameters)
  optim.start<-c(100,rep(0.5,2))
  optim.up<-c(2000,rep(1,2))
  optim.down<-c(10,rep(0.01,2))
  
  #ML for sexual selection, 3 parameters
  model.s<-optim(par=optim.start,fn = logL.all,method = "L-BFGS-B", control = list(fnscale=-1),lower = optim.down,upper = optim.up,N=N,mode="s",hessian = TRUE)
  #calculate CI for estimated parameters
  ci.s<-calculate.ci(model.s$hessian)
  upper.s<-model.s$par+ci.s
  lower.s<-model.s$par-ci.s
  
  #ML for viability, 3 parameters
  model.v<-optim(par=optim.start,fn = logL.all,method = "L-BFGS-B", control = list(fnscale=-1),lower = optim.down,upper = optim.up,N=N,mode="v",hessian = TRUE)
  #calculate CI for estimated parameters
  ci.v<-calculate.ci(model.v$hessian)
  upper.v<-model.v$par+ci.v
  lower.v<-model.v$par-ci.v
  
  #ML for fecundity, 3 parameters
  model.f<-optim(par=optim.start,fn = logL.all,method = "L-BFGS-B", control = list(fnscale=-1),lower = optim.down,upper = optim.up,N=N,mode="f",hessian = TRUE)
  #calculate CI for estimated parameters
  ci.f<-calculate.ci(model.f$hessian)
  upper.f<-model.f$par+ci.f
  lower.f<-model.f$par-ci.f
  
  #generate data frame and store results
  res<-data.frame("sim"=rep(sim.nr,3),"est.ne"=c(model.s$par[1],model.v$par[1],model.f$par[1]),"est.p1"=c(model.s$par[2],model.v$par[2],model.f$par[2]),
                  "est.p2"=c(model.s$par[3],model.v$par[3],model.f$par[3]), "ci.ne.upper"=c(upper.s[1],upper.v[1],upper.f[1]), 
                  "ci.ne.lower"=c(lower.s[1],lower.v[1],lower.f[1]), "ci.p1.upper"=c(upper.s[2],upper.v[2],upper.f[2]),
                  "ci.p1.lower"=c(lower.s[2],lower.v[2],lower.f[2]), "ci.p2.upper"=c(upper.s[3],upper.v[3],upper.f[3]),
                  "ci.p2.lower"=c(lower.s[3],lower.v[3],lower.f[3]),
                  "L"=c(model.s$value,model.v$value,model.f$value),"model"=c("s","v","f"),"data"=rep(info,3))
  
  #set start + limits for ML, 2 parameters estimated (1 Ne + 1 selection parameter)
  optim.start<-c(100,0.5)
  optim.up<-c(2000,1)
  optim.down<-c(10,0.01)
   
  #ML for sexual selection, 2 parameters
  model.s.simple<-optim(par=optim.start,fn = logL.all,method = "L-BFGS-B", control = list(fnscale=-1),lower = optim.down,upper = optim.up,N=N,mode="s.simple",hessian = TRUE)
  #calculate CI for estimated parameters
  ci.s.simple<-calculate.ci(model.s.simple$hessian)
  upper.s.simple<-model.s.simple$par+ci.s.simple
  lower.s.simple<-model.s.simple$par-ci.s.simple
    
  #ML for viability, 2 parameters
  model.v.simple<-optim(par=optim.start,fn = logL.all,method = "L-BFGS-B", control = list(fnscale=-1),lower = optim.down,upper = optim.up,N=N,mode="v.simple",hessian = TRUE)
  #calculate CI for estimated parameters
  ci.v.simple<-calculate.ci(model.v.simple$hessian)
  upper.v.simple<-model.v.simple$par+ci.v.simple
  lower.v.simple<-model.v.simple$par-ci.v.simple
  
  #ML for fecundity, 2 parameters
  model.f.simple<-optim(par=optim.start,fn = logL.all,method = "L-BFGS-B", control = list(fnscale=-1),lower = optim.down,upper = optim.up,N=N,mode="f.simple",hessian = TRUE)
  #calculate CI for estimated parameters
  ci.f.simple<-calculate.ci(model.f.simple$hessian)
  upper.f.simple<-model.f.simple$par+ci.f.simple
  lower.f.simple<-model.f.simple$par-ci.f.simple
  

  #ML for fecundity, type 2, 2 parameters
  model.f.simple2<-optim(par=optim.start,fn = logL.all,method = "L-BFGS-B", control = list(fnscale=-1),lower = optim.down,upper = optim.up,N=N,mode="f.simple2",hessian = TRUE)
  #calculate CI for estimated parameters
  ci.f.simple2<-calculate.ci(model.f.simple2$hessian)
  upper.f.simple2<-model.f.simple2$par+ci.f.simple2
  lower.f.simple2<-model.f.simple2$par-ci.f.simple2
  
  #generate data frame and safe result 
  res.simple<-data.frame("sim"=rep(sim.nr,4),"est.ne"=c(model.s.simple$par[1],model.v.simple$par[1],model.f.simple$par[1],model.f.simple2$par[1]),"est.p1"=c(model.s.simple$par[2],model.v.simple$par[2],model.f.simple$par[2],model.f.simple2$par[2]),"est.p2"=rep(NA,4),
                    "ci.ne.upper"=c(upper.s.simple[1],upper.v.simple[1],upper.f.simple[1],upper.f.simple2[1]), 
                    "ci.ne.lower"=c(lower.s.simple[1],lower.v.simple[1],lower.f.simple[1],lower.f.simple2[1]), "ci.p1.upper"=c(upper.s.simple[2],upper.v.simple[2],upper.f.simple[2],upper.f.simple2[2]),
                    "ci.p1.lower"=c(lower.s.simple[2],lower.v.simple[2],lower.f.simple[2],lower.f.simple2[2]), "ci.p2.upper"=rep(NA,4),"ci.p2.lower"=rep(NA,4),
                    "L"=c(model.s.simple$value,model.v.simple$value,model.f.simple$value,model.f.simple2$value),"model"=c("s.simple","v.simple","f.simple","f.simple2"),"data"=rep(info,4))
  #combine data frames and return
  res<-rbind(res,res.simple)
  return(res)
}

#call.cross: function to call function cross and set selection parameters 

call.cross<-function(n.e,p1,p2,x,n.rep,info,n.sim,g){
  v.m=c(1,1)
  v.f=c(1,1)
  w=matrix(rep(1,9),ncol=3)
  alpha=c(1,1)
  beta=c(1,1)
  gamma=c(1,1)
  
  if(info=="s"){
    alpha[2]<-p1
    beta[2]<-alpha[2]
    gamma[2]<-p2
  }
  if(info=="v"){
    v.m[2]<-p1
    v.f[2]<-p2
  }
  if(info=="f"){
    w[3,1]<-p1
    w[1,3]<-p2
    w[3,2]<-w[3,1]
    w[2,3]<-w[1,3]
    w[3,3]<-w[1,3]*w[3,1]
  }
  
  if(info =="v.simple"){ 
    v.m[2]<-p1
    v.f[2]<-v.m[2]
  }
  
  if(info=="f.simple"){ 
    w[3,1]<-p1
    w[1,3]<-w[3,1]
    w[3,2]<-w[3,1]
    w[2,3]<-w[3,1]
    w[3,3]<-w[1,3]*w[3,1]
  }
  
  if(info=="f.simple2"){ 
    w[1,3]<-p1
    w[2,3]<-w[1,3]
    w[3,3]<-w[1,3]
  }
  
  if(info=="s.simple") {
    alpha[2]<-p1
    beta[2]<-alpha[2]
    gamma[2]<-alpha[2]
  }
  
  trial<-sapply(c(1:n.sim),function(iter) cross(x=x,n.e=n.e,v.m=v.m,v.f=v.f,alpha = alpha,beta=beta,gamma=gamma,w=w,g=g,n.rep=n.rep,info=info,sim.nr=iter),simplify = F)
  trial<-do.call("rbind",trial)
  return(trial)
}


##################################
##################################


#read parameters from command line
args = commandArgs(trailingOnly=TRUE)
params <- read.table(args[1],header=F,sep = "\t")
colnames(params)<-c("n.e","p1","p2")
g <- as.integer(args[2])
n.rep <- as.integer(args[3])
n.sim <- as.integer(args[4])
iter <- as.integer(args[5])
params<-params[iter,]
print(params)

########################################################
#        starting allele frequencies, 70 %, no HET     #
########################################################
x<-c(0.3,0,0.7,0.3,0,0.7)
start<-Sys.time()
##################################
# fecundity                      #
##################################

trial<-call.cross(x=x,n.e = params$n.e,p1 = params$p1,p2 = params$p2,n.sim = n.sim,n.rep = n.rep,info = "f",g = g)
trial$n.e<-rep(params$n.e,nrow(trial))
trial$p1<-rep(params$p1,nrow(trial))
trial$p2<-rep(params$p2,nrow(trial))
tosave<-paste("p1",params$p1,"p2",params$p2,"ne",params$n.e,"reps",n.rep,"gens",g,"data",info,sep="-")
tosave<-paste(tosave,".rds",sep = "")
print(tosave)
saveRDS(trial,file =paste(save.dir,tosave,sep = "/"))

##################################
# viability                      #
##################################
trial<-call.cross(x=x,n.e = params$n.e,p1 = params$p1,p2 = params$p2,n.sim = n.sim,n.rep = n.rep,info = "v",g = g)
trial$n.e<-rep(params$n.e,nrow(trial))
trial$p1<-rep(params$p1,nrow(trial))
trial$p2<-rep(params$p2,nrow(trial))
tosave<-paste("p1",params$p1,"p2",params$p2,"ne",params$ne,"reps",n.rep,"gens",g,"data",info,sep="-")
tosave<-paste(tosave,".rds",sep = "")
print(tosave)
saveRDS(trial,file =paste(save.dir,tosave,sep = "/"))

##################################
# sexual Selection               #
##################################
trial<-call.cross(x=x,n.e = params$n.e,p1 = params$p1,p2 = params$p2,n.sim = n.sim,n.rep = n.rep,info = "s",g = g)
trial$n.e<-rep(params$n.e,nrow(trial))
trial$p1<-rep(params$p1,nrow(trial))
trial$p2<-rep(params$p2,nrow(trial))
tosave<-paste("p1",params$p1,"p2",params$p2,"ne",params$ne,"reps",n.rep,"gens",g,"data",info,sep="-")
tosave<-paste(tosave,".rds",sep = "")
print(tosave)
saveRDS(trial,file =paste(save.dir,tosave,sep = "/"))
end<-Sys.time()
print(start-end)

